<?php

namespace Modules\Cotacao\Modules\ListaPreco\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use \Setforms2\Shared\ORM\DateTime;
use Exception;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Sup01200MontagemListaPrecoController extends \Teknisa\GER\Controllers\TeknisaCrud
{

    protected $crudService;
    protected $service;
    protected $connection;
    protected $newCode;
    protected $dataSourceName;
    protected $dataSourceNameToSave;

    public function __construct($crudService, $parameterBag, $connection, $environment, $newCode, $service)
    {
        $this->crudService  = $crudService;
        $this->connection   = $connection;
        $this->environment  = $environment;
        $this->newCode      = $newCode;
        $this->service      = $service;
        parent::__construct($crudService, $parameterBag);
    }

    public function confirmaSenhaComprador(DTO\Request\Filter $request, DTO\Response $response)
    {
        $this->dataSourceName = 'operador';

        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $conditions = array_filter($filter, function ($row) {
                return $row['columnName'] === 'CDOPERADOR';
            }, ARRAY_FILTER_USE_BOTH);
            $operador = $this->crudService->getFilteredDataSet('operador', $conditions)->getRows();

            $passWord = array_filter($filter, function ($row) {
                return $row['columnName'] === 'PASSWORD';
            }, ARRAY_FILTER_USE_BOTH);

            if (isset(end($operador)['CDSENHAOPERWEB']) && (end($operador)['CDSENHAOPERWEB'] !== md5(end($passWord)['value']))) {
                throw new Exception("INVALID_PASSWORD");
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function validaData(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $conditions = $filterCriteria->getConditions();
            $dataset = $this->crudService->getFilteredDataSet("sup01200_validaData", $conditions)->getRows();
            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    private function getCentral(&$row)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'CDFILIAL',
                    'operator' => '=',
                    "value" => $this->environment->getCdFilial()
                )
            );

            $result = $this->crudService->getFilteredDataSet("filial", $conditions)->getRows();
            if (isset($result) && count($result)) {
                $filial = end($result);
                $row['P_CDFILIAL'] = isset($filial['CDFILCENTCMP']) ? $filial['CDFILCENTCMP'] : $filial['CDFILIAL'];
            } else {
                throw new \Exception('CDFILCENTCMP_NOT_REGISTERED');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getNewCode(&$row)
    {
        try {
            if (!isset($row['P_NRSEQPRELIST']) || empty($row['P_NRSEQPRELIST'])) {
                $row['P_NRSEQPRELIST'] = $this->newCode->getCodeByTableName('PREVLISTA' . $row['P_CDFILIAL'], $this->environment->getNrorgTrab(), 1, 10, false);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getExistsPrevLista($row)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_CDFILIAL',
                    'operator' => '=',
                    "value" => $row['P_CDFILIAL']
                ),
                array(
                    'columnName' => 'P_NRSEQPRELIST',
                    'operator' => '=',
                    "value" => $row['P_NRSEQPRELIST']
                )
            );

            $result = $this->crudService->getFilteredDataSet("sup01200_existsPrevlista", $conditions)->getRows();
            return isset($result) && count($result);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getParamgeral(&$row)
    {
        try {
            $result = $this->crudService->getFilteredDataSet("paramgeral", array())->getRows();
            if (isset($result) && count($result)) {
                $paramgeral = end($result);
                $row['IDSISDOS'] = isset($paramgeral['IDSISDOS']) ? $paramgeral['IDSISDOS'] : '1';
            } else {
                throw new \Exception('PARAMGERAL_NOT_REGISTERED');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insMontaItmPrevListGR(&$row)
    {
        try {
            $procedure = new StoredProcedure($this->connection, 'INS_MONTA_ITMPREVLIST');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT,  $row['P_CDFILIAL'],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT,  $row['P_CDOPERADOR'],      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TCDREGIAO',    Param::PARAM_INPUT,  $row['P_CDREGIAO'],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',     Param::PARAM_INPUT,  $row['P_CDREGIAO'],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINI',        Param::PARAM_INPUT,  $row['P_DTINIVALCOL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFIN',        Param::PARAM_INPUT,  $row['P_DTFINVALCOL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TPRODUTO',     Param::PARAM_INPUT,  $row['P_TPRODUTO'],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT,  $row['P_CDARVPRODINI'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT,  $row['P_CDARVPRODFIN'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CARGCOT',      Param::PARAM_INPUT,  $row['P_IDCOTSEMMOVLIST'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT,  $row['P_NRSEQPRELIST'],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_COINCIDEP',    Param::PARAM_INPUT,  $row['P_IDCOINVALIDLIST'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT',   Param::PARAM_OUTPUT, NULL,                      Param::PARAM_TYPE_STR, 20));
            $data = $procedure->execute();
            $row['ROWSAFFECT'] = is_array($data) && count($data) && $data['P_ROWSAFFECT'] ? $data['P_ROWSAFFECT'] : 0;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getRegiao($row)
    {
        try {
            if (isset($row['P_CDREGIAO']) && $row['P_CDREGIAO'] === 'T') {
                $dataSourceName = 'regiao';
                $conditions = array();
            } else {
                $dataSourceName = 'regiaoInstrQuery';
                $conditions = array(
                    array(
                        'columnName' => 'P_CDREGIAO',
                        'operator' => '=',
                        "value" => $row['P_CDREGIAO']
                    )
                );
            }

            return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getCountItmprevlist(&$row)
    {
        try {
            $conditions = array(
                array(
                    'columnName' => 'P_CDFILIAL',
                    'operator' => '=',
                    "value" => $row['P_CDFILIAL']
                ),
                array(
                    'columnName' => 'P_NRSEQPRELIST',
                    'operator' => '=',
                    "value" => $row['P_NRSEQPRELIST']
                )
            );

            $result = $this->crudService->getFilteredDataSet("sup01200_countItmprevlist", $conditions)->getRows();

            if (isset($result) && count($result)) {
                $row['ROWSAFFECT'] = end($result)['CONTADOR'];
            } else {
                $row['ROWSAFFECT'] = 0;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insMontaItmPrevList(&$row)
    {
        try {
            $regioes = $this->getRegiao($row);

            foreach ($regioes as $regiao) {
                $this->service->insertItmprevlist($row, $regiao['CDREGIAO']);
            }

            $this->service->deleteItmprevlistNotExists($row);
            $this->getCountItmprevlist($row);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function montarLista(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();

        try {
            $this->getCentral($row);
            $this->getNewCode($row);
            $this->getParamgeral($row);

            $data = array(
                'CDFILCENTCMP' => $row['P_CDFILIAL'],
                'NRSEQPRELIST' => $row['P_NRSEQPRELIST'],
                'IDSISDOS'     => $row['IDSISDOS'],
            );

            $row['P_TPRODUTO']     = isset($row['P_CDARVPRODINI']) && !empty($row['P_CDARVPRODINI']) ? null : 'T';
            $row['P_CDARVPRODINI'] = isset($row['P_CDARVPRODINI']) ? $row['P_CDARVPRODINI'] : null;
            $row['P_CDARVPRODFIN'] = isset($row['P_CDARVPRODFIN']) ? str_pad($row['P_CDARVPRODFIN'], 20, 'z', STR_PAD_RIGHT) : null;

            if ($this->getExistsPrevLista($row)) {
                $this->service->updatePrevLista($row);
            } else {
                $this->service->insertPrevlista($row);
            }

            if ($row['IDSISDOS'] === '3') {
                $this->insMontaItmPrevListGR($row);
            } else {
                $this->insMontaItmPrevList($row);
            }

            $data = array_merge($data, array('ROWSAFFECT' => $row['ROWSAFFECT']));

            if ($row['ROWSAFFECT'] === 0) {
                $this->service->updateItemcole($row);
                $this->service->deletePrevlist($row);
            }

            $response->addDataSet(new DataSet('data', $data));
        } catch (\Exception $e) {
            if (isset($row['P_CDFILIAL']) && isset($row['P_NRSEQPRELIST'])) {
                $this->service->deleteItmprevlist($row);
            }
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getLista(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $filterCriteria->addCondition('P_CDFILIAL', $this->environment->getCdFilial());
            $dataset = $this->crudService->getFilteredDataSet("sup01200_lista", $filterCriteria->getConditions())->getRows();
            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveItmprevlist(DTO\Request $request, DTO\Response $response)
    {
        try {
            $this->dataSourceNameToSave = 'itmprevlist';
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    private function createFilterCriteria($params)
    {
        $filterCriteria = new FilterCriteria($this->dataSourceName);

        foreach ($params as $name => $value) {
            $filterCriteria->addCondition($name, '=', $value);
        }

        return $filterCriteria;
    }

    private function getCotacoes($row, $request, &$response)
    {
        try {
            $this->dataSourceName = 'sup01200_existsCotacao';
            $params = array(
                'P_CDFILIAL'     => $row['P_CDFILIAL'],
                'P_NRSEQPRELIST' => $row['P_NRSEQPRELIST']
            );
            $filterCriteria = $this->createFilterCriteria($params);
            $newRequest = new DTO\Request\Filter(
                $filterCriteria,
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            );
            parent::find($newRequest, $response);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getCotacaoData(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $this->getCotacoes($row, $request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function selecCotAuto(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $procedure = new StoredProcedure($this->connection, 'APURA_LISTA');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $row['P_CDFILIAL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT, $row['P_NRSEQPRELIST'], Param::PARAM_TYPE_STR));
            $procedure->execute();

            $this->getCotacoes($row, $request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteItmprevlist(DTO\Request $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = 'itmprevlist';
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function limpaItemcoleMontagem(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            $prodIni = isset($row["P_CDARVPRODINI"]) ? $row["P_CDARVPRODINI"] : null;
            $prodFin = isset($row["P_CDARVPRODFIN"]) ? $row["P_CDARVPRODFIN"] : null;
            $tProduto = isset($row["P_CDARVPRODINI"]) && isset($row["P_CDARVPRODFIN"]) ? null : "T";
            $carGot = $row["P_IDCOTSEMMOVLIST"] === 'S' ? 'T' : null;

            $procedure = new StoredProcedure($this->connection, 'LIMPA_ITEMCOLE_MONTAGEM');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT,  $row["P_CDFILIAL"],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINIVALITM',  Param::PARAM_INPUT,  $row["P_DTINIVALITM"],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINVALITM',  Param::PARAM_INPUT,  $row["P_DTFINVALITM"],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDREGIAO',     Param::PARAM_INPUT,  $row["P_CDREGIAO"],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TCDREGIAO',    Param::PARAM_INPUT,  $row["P_CDREGIAO"],        Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT,  $prodIni,                  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT,  $prodFin,                  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TPRODUTO',     Param::PARAM_INPUT,  $tProduto,                 Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CARGCOT',      Param::PARAM_INPUT,  $carGot,                   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT,  $row["P_CDOPERADOR"],      Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_COINCIDE',     Param::PARAM_INPUT,  $row["P_IDCOINVALIDLIST"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT,  $row["P_NRSEQPRELIST"],    Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_PREENCHETAB',  Param::PARAM_INPUT,  $row["P_PREENCHETAB"],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT',   Param::PARAM_OUTPUT, null,                      Param::PARAM_TYPE_STR, 20));
            $result = $procedure->execute();
            return $result["P_ROWSAFFECT"];
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function criarLista(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            $procedure = new StoredProcedure($this->connection, 'CRIA_LISTA');
            $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT,  $row['P_CDFILIAL'],     Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT,  $row['P_NRSEQPRELIST'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ROWSAFFECT',   Param::PARAM_OUTPUT, NULL,                   Param::PARAM_TYPE_STR, 20));
            $data = $procedure->execute();

            $response->addDataSet(new DataSet('data', $data));

            // $qtdDone = 0;
            // $qtdToDo = $row['P_QTDTODO'];

            // while ($qtdDone < $qtdToDo) {
            //     var_dump(1);die;
            //     $data = $procedure->execute();            
            //     $qtdDone = $qtdDone + $data["P_ROWSAFFECT"];
            //     $response->addDataSet(new DataSet('data', array("DONE", $qtdDone)));
            // }

            //$response->addDataSet(new DataSet('data', array("QTDDONE", $qtdDone)));


            /*$oleResult = array();
            switch ($row["P_OPCAO"]) {
                
                case 2:
                    $procedure = new StoredProcedure($this->connection, 'CRIA_LISTA');
                    $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT,  $row['P_CDFILIAL'],     Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('P_NRSEQPRELIST', Param::PARAM_INPUT,  $row['P_NRSEQPRELIST'], Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('P_ROWSAFFECT',   Param::PARAM_OUTPUT, NULL,                   Param::PARAM_TYPE_STR, 20));
                    $rowsaffect = $procedure->execute();
                    //$oleResult[2] = $rowsaffect["P_ROWSAFFECT"];
                    return $rowsaffect["P_ROWSAFFECT"];
                    break;
                case 3:
                    $rowsaffect = self::adsLimpaItens(
                        $row['P_CDFILIAL'],
                        $row['P_DTINIVALITM'],
                        $row['P_DTFINVALITM'],
                        $row['P_CDREGIAO'],
                        $row['P_TCDREGIAO'],
                        $row['P_CDPRODINI'],
                        $row['P_CDPRODFIN'],
                        $row['P_TPRODUTO'],
                        $row['P_CARGCOT'],
                        $row['P_CDOPERADOR'],
                        $row['P_NRSEQPRELIST'],
                        'S',
                        $row["P_COINCIDE"]
                    );
                    return $rowsaffect;
                    //$oleResult[3] = $rowsaffect["P_ROWSAFFECT"];
                    break;
                case 4:
                    $rowsaffect = self::adsLimpaItens(
                        $row['P_CDFILIAL'],
                        $row['P_DTINIVALITM'],
                        $row['P_DTFINVALITM'],
                        $row['P_CDREGIAO'],
                        $row['P_TCDREGIAO'],
                        $row['P_CDPRODINI'],
                        $row['P_CDPRODFIN'],
                        $row['P_TPRODUTO'],
                        $row['P_CARGCOT'],
                        $row['P_CDOPERADOR'],
                        $row['P_NRSEQPRELIST'],
                        'N',
                        $row["P_COINCIDE"]
                    );
                    return $rowsaffect;
                    //$oleResult[4] = $rowsaffect["P_ROWSAFFECT"];
                    break;
                }*/
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    //aqui













    public function gerarListaPrecos(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $oleResult = $row['oleResult'];
            $stLock = $row['stLock'];

            if ($row['boConfirma']) {
                $pos = 0;
                $row['P_OPCAO'] = 2;
                $inQtdeExistente = $oleResult[1];
                while (($inQtdeExistente - $pos) > 0) {
                    $reqRow = new DTO\Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                    $oleResult[2] = self::criarLista($reqRow, $response);
                    $pos = $pos + $oleResult[2];
                    //$oleResult = $response['oleResult'];
                }
                $row['P_OPCAO'] = 3;
                $reqRow = new DTO\Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $oleResult[3] = self::criarLista($reqRow, $response);
                //$oleResult = $response['oleResult'];
                $pos = 0;
                $row['P_OPCAO'] = 4;
                while (($oleResult[3] - $pos) > 0) {
                    $reqRow = new DTO\Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                    $oleResult[4] = self::criarLista($reqRow, $response);
                    //$oleResult = $response['oleResult'];
                    $pos = $pos + $oleResult[4];
                }

                self::finalizaLista($row['P_CDFILIAL'], $row['P_NRSEQPRELIST'], $stLock);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }









    public function adqLock($P_CDFILIAL, $P_DTINIVALITM, $P_DTFINVALITM, $P_CDREGIAO, $P_TCDREGIAO, $P_CDPRODINI, $P_CDPRODFIN, $P_TPRODUTO, $P_CDOPERADOR)
    {
        $query = "SELECT CDOPERADOR, NRSEQLOCKLIST FROM LOCK_LISTA WHERE CDFILIAL = :P_CDFILIAL AND((DTINIVALITMLOCK <= :P_DTINIVALITM AND DTFINVALITMLOCK >= :P_DTFINVALITM) OR (DTINIVALITMLOCK >= :P_DTINIVALITM AND DTFINVALITMLOCK <= :P_DTFINVALITM)) AND ((INSTR(DSREGIAOLOCK, :P_CDREGIAO) > 0) OR (INSTR(:P_CDREGIAO, DSREGIAOLOCK) > 0) OR (:P_TCDREGIAO = 'T') OR (IDTCDREGIAOLOCK = 'T') ) AND ((CDPRODINILOCK >= :P_CDPRODINI AND CDPRODFINLOCK <= :P_CDPRODFIN) OR (CDPRODINILOCK <= :P_CDPRODINI AND CDPRODFINLOCK >= :P_CDPRODFIN) OR (:P_TPRODUTO = 'T') OR (IDTPRODUTOLOCK = 'T') ) AND CDCOMPRADOR = :P_CDOPERADOR";
        $stmt = $this->connection->prepare($query);
        $stmt->execute(array(
            "P_CDFILIAL" => $P_CDFILIAL,
            "P_DTINIVALITM" => $P_DTINIVALITM,
            "P_DTFINVALITM" => $P_DTFINVALITM,
            "P_CDREGIAO" => $P_CDREGIAO,
            "P_TCDREGIAO" => $P_TCDREGIAO,
            "P_CDPRODINI" => $P_CDPRODINI,
            "P_CDPRODFIN" => $P_CDPRODFIN,
            "P_TPRODUTO" => $P_TPRODUTO,
            "P_CDOPERADOR" => $P_CDOPERADOR
        ));
        return $stmt->fetchAll();
    }

    public function acmLock($P_PRXLOCK, $P_CDFILIAL, $P_DTINIVALITM, $P_DTFINVALITM, $P_CDREGIAO, $P_TCDREGIAO, $P_CDPRODINI, $P_CDPRODFIN, $P_TPRODUTO, $P_CDCOMPRADOR, $P_CDOPERADOR)
    {
        $query = "INSERT INTO LOCK_LISTA( NRSEQLOCKLIST, CDFILIAL, DTINIVALITMLOCK, DTFINVALITMLOCK, DSREGIAOLOCK, IDTCDREGIAOLOCK, CDPRODINILOCK, CDPRODFINLOCK, IDTPRODUTOLOCK, CDCOMPRADOR, CDOPERADOR) VALUES (:P_PRXLOCK,:P_CDFILIAL,:P_DTINIVALITM,:P_DTFINVALITM, :P_CDREGIAO,:P_TCDREGIAO,:P_CDPRODINI,:P_CDPRODFIN, :P_TPRODUTO,:P_CDCOMPRADOR, :P_CDOPERADOR)";
        $stmt = $this->connection->prepare($query);
        $stmt->execute(array(
            "P_PRXLOCK" => $P_PRXLOCK,
            "P_CDFILIAL" => $P_CDFILIAL,
            "P_DTINIVALITM" => $P_DTINIVALITM,
            "P_DTFINVALITM" => $P_DTFINVALITM,
            "P_CDREGIAO" => $P_CDREGIAO,
            "P_TCDREGIAO" => $P_TCDREGIAO,
            "P_CDPRODINI" => $P_CDPRODINI,
            "P_CDPRODFIN" => $P_CDPRODFIN,
            "P_TPRODUTO" => $P_TPRODUTO,
            "P_CDCOMPRADOR" => $P_CDCOMPRADOR,
            "P_CDOPERADOR" => $P_CDOPERADOR
        ));
    }

    public function adqCountItmPrevList($P_CDFILIAL, $P_NRSEQPRELIST)
    {
        $query = "SELECT COUNT(*) AS CONTADOR FROM ITMPREVLIST WHERE(NRSEQPRELIST = :P_NRSEQPRELIST) AND (CDFILIAL = :P_CDFILIAL)";
        $stmt = $this->connection->prepare($query);
        $stmt->execute(array(
            "P_CDFILIAL" => $P_CDFILIAL,
            "P_NRSEQPRELIST" => $P_NRSEQPRELIST
        ));
        $result = $stmt->fetchAll();

        return $result[0]["CONTADOR"];
    }


    public function adsLimpaItens($P_CDFILIAL, $P_DTINIVALITM, $P_DTFINVALITM, $P_CDREGIAO, $P_TCDREGIAO, $P_CDPRODINI, $P_CDPRODFIN, $P_TPRODUTO, $P_CARGCOT, $P_CDOPERADOR, $P_NRSEQPRELIST, $P_PREENCHETAB, $P_COINCIDE)
    {
        $p_coincide = ($P_COINCIDE) ? 'S' : 'N';

        $connection = $this->connection;
        $procedure = new StoredProcedure($connection, 'LIMPA_ITEMCOLE_MONTAGEM');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $P_CDFILIAL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINIVALITM',    Param::PARAM_INPUT, $P_DTINIVALITM, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINVALITM',    Param::PARAM_INPUT, $P_DTFINVALITM, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDREGIAO',    Param::PARAM_INPUT, $P_CDREGIAO, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TCDREGIAO',    Param::PARAM_INPUT, $P_TCDREGIAO, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT, $P_CDPRODINI, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT, $P_CDPRODFIN, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TPRODUTO',    Param::PARAM_INPUT, $P_TPRODUTO, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CARGCOT',    Param::PARAM_INPUT, $P_CARGCOT, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR',    Param::PARAM_INPUT, $P_CDOPERADOR, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQPRELIST',    Param::PARAM_INPUT, $P_NRSEQPRELIST, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PREENCHETAB',    Param::PARAM_INPUT, $P_PREENCHETAB, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_COINCIDE',    Param::PARAM_INPUT, $p_coincide, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ROWSAFFECT',  Param::PARAM_OUTPUT, NULL,         Param::PARAM_TYPE_STR, 20));
        $rowsaffect = $procedure->execute();
        return $rowsaffect["P_ROWSAFFECT"];
    }





    public function qryOperador(DTO\Request\Row $request, DTO\Response $response)
    {

        $row = $request->getRow();

        $query = "SELECT OPERADOR.CDOPERADOR, OPERADOR.NMOPERADOR FROM OPERADOR WHERE OPERADOR.CDOPERADOR = :P_CDOPERADOR";
        $stmt = $this->connection->prepare($query);
        $stmt->execute(array(
            "P_CDOPERADOR" => $row['P_CDOPERADOR']
        ));
        $data = $stmt->fetchAll();
        $response->addDataSet(new DataSet('data', $data));
    }

    public function finalizaLista($P_CDFILIAL, $P_NRSEQPRELIST, $P_NRLOCK)
    {
        try {
            $query = "UPDATE PREVLISTA SET IDDISPLISFIL = 'N' WHERE CDFILIAL = :P_CDFILIAL AND NRSEQPRELIST = :P_NRSEQPRELIST";
            $stmt = $this->connection->prepare($query);
            $stmt->execute(array(
                "P_CDFILIAL" => $P_CDFILIAL,
                "P_NRSEQPRELIST" => $P_NRSEQPRELIST
            ));

            $query = "UPDATE ITMPREVLIST SET IDITMLISTJAMONT = 'N' WHERE CDFILIAL = :P_CDFILIAL AND NRSEQPRELIST = :P_NRSEQPRELIST ";
            $stmt = $this->connection->prepare($query);
            $stmt->execute(array(
                "P_CDFILIAL" => $P_CDFILIAL,
                "P_NRSEQPRELIST" => $P_NRSEQPRELIST
            ));

            if ($P_NRLOCK) {
                $query = "DELETE FROM LOCK_LISTA WHERE NRSEQLOCKLIST = :P_PRXLOCK";
                $stmt = $this->connection->prepare($query);
                $stmt->execute(array(
                    "P_PRXLOCK" => $P_NRLOCK
                ));
            }
        } catch (Exception $e) {
            throw $e;
        }
    }
}
