<?php
namespace Modules\Cotacao\Modules\ListaPreco\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class Sup05500ListaPreco extends \Zeedhi\Framework\Controller\Report {
    
    protected $entityManager;
    protected $qr2Strategy;
    protected $birtStrategy;
    protected $newCode;
    protected $environment;
    protected $connection;
    protected $dataSourceManager;
    
    public function __construct($reportService, $entityManager, $qr2Strategy,$birtStrategy, $newCode, $environment, $connection, $dataSourceManager) {
        $this->entityManager     = $entityManager;
        $this->qr2Strategy       = $qr2Strategy;
        $this->birtStrategy      = $birtStrategy;
        $this->reportService     = $reportService;
        $this->newCode           = $newCode;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->dataSourceManager = $dataSourceManager;
    }
    
    protected $reportMapping = array ();
     public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row['__report_name'] = str_replace(".QR2","",$row['REPORT']);
        $row['__report_name'] = str_replace(".BIRT","",$row['REPORT']);
        $row['CDFILIAL'] = $this->environment->getCdFilial();
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        
        if($row['STRATEGY'] == 'BIRT'){
        $row['BIRTTYPE'] = $row['QR2TYPE'];
        unset($row['QR2TYPE']);
        }
        
        $this->stNomeRel = $this->getReportName($row);
        $row['NRORG'] = $this->environment->getNrOrg();
        $row['DTINI'] = $row['DTINTERVAL']['start'];
        $row['DTFIN'] = $row['DTINTERVAL']['end'];
        
        if($row['STRATEGY'] == 'BIRT'){
            if($row['CDCOMPRADOR'] !== "T"){
                $nrlistaForn = $this->criarListaValores($row['CDCOMPRADOR']);
                $row['CDCOMPRADOR'] = "_NRLISTA=".$nrlistaForn;
            }
            if($row['NRSEQPRELIST'] !== "T"){
                $nrlistaForn = $this->criarListaValores($row['NRSEQPRELIST']);
                $row['NRSEQPRELIST'] = "_NRLISTA=".$nrlistaForn;
            }
            if($row['CDOPERADOR'] !== "T"){
                $nrlistaForn = $this->criarListaValores($row['CDOPERADOR']);
                $row['CDOPERADOR'] = "_NRLISTA=".$nrlistaForn;
            }
        }
        
        
        $this->reportMapping[$this->stNomeRel] =
            array (
                'strategy' => $row['STRATEGY'],
                'parameterMapping' =>
                    array (
                        "CDFILIAL"      => "P_CDFILIAL",
                        "DTINI"         => "P_DTINI",
                        "DTFIN"         => "P_DTFIN",
                        "NRSEQPRELIST"  => "L_LISTAPRECO",
                        "CDCOMPRADOR"   => "L_CDCOMPRADOR",
                        "CDOPERADOR"    => "L_CDOPERADOR",
                        "NRORG"         => "P_NRORG"
                    ),
            );
}
    
        protected function criarListaValores($lista){
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
        $valores = explode('_',$lista);
        try{
            foreach($valores as $key){
                $this->insertLista($nrlista, $key);
            }
            return $nrlista;
        } catch(\Exception $e){
            throw $e;
        }
    }

    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );

        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

}